package fi.hsivonen.photobackup;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import nu.validator.xml.EmptyAttributes;

import org.apache.xml.serializer.Method;
import org.apache.xml.serializer.OutputPropertiesFactory;
import org.apache.xml.serializer.Serializer;
import org.apache.xml.serializer.SerializerFactory;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import com.aetrion.flickr.Flickr;
import com.aetrion.flickr.FlickrException;
import com.aetrion.flickr.RequestContext;
import com.aetrion.flickr.auth.Auth;
import com.aetrion.flickr.auth.Permission;
import com.aetrion.flickr.util.AuthStore;
import com.aetrion.flickr.util.FileAuthStore;

/**
 * @author Matthew MacKenzie
 * @author hsivonen
 */

public class Main {

    private final String nsid;
    private final Flickr flickr;
    private final AuthStore authStore;
    private final String sharedSecret;
    private final FlickrEntityResolver resolver;
    private final File metadataDir;
    private final File photoDir;
    private final File tempDir;
    private final Map<String, String> photourls = new HashMap<String, String>(); 

    public Main(String apiKey, String nsid, String sharedSecret, File rootDir)
      throws IOException {
        if (!rootDir.isDirectory()) {
            throw new IOException(rootDir.getCanonicalPath() + "is not a directory.");
        }
        this.flickr = new Flickr(apiKey);
        this.sharedSecret = sharedSecret;
        this.nsid = nsid;       
        this.flickr.setSharedSecret(sharedSecret);        
        this.authStore = new FileAuthStore(new File(rootDir, "authentication"));
        this.metadataDir = new File(rootDir, "metadata");
        this.photoDir = new File(rootDir, "photos");
        this.tempDir = new File(rootDir, "tmp");
        if (!metadataDir.exists()) {
            metadataDir.mkdir();
        }
        if (!photoDir.exists()) {
            photoDir.mkdir();
        }
        if (!tempDir.exists()) {
            tempDir.mkdir();
        }
        this.resolver = new FlickrEntityResolver(apiKey, nsid, sharedSecret, tempDir);
    }

	private void authorize() throws IOException, SAXException, FlickrException {
		String frob = this.flickr.getAuthInterface().getFrob();

		URL authUrl = this.flickr.getAuthInterface().buildAuthenticationUrl(Permission.READ, frob);
		System.out.println("Please visit: " + authUrl.toExternalForm() + " then, hit enter.");

		System.in.read();

		Auth token = this.flickr.getAuthInterface().getToken(frob);
		RequestContext.getRequestContext().setAuth(token);
		this.authStore.store(token);
		System.out.println("Thanks.  You probably will not have to do this every time.  Now starting backup.");
	}
	
	public void doBackup() throws Exception {
		
		RequestContext rc = RequestContext.getRequestContext();
		rc.setSharedSecret(this.sharedSecret);
		
		if (this.authStore != null) {
			Auth auth = this.authStore.retrieve(this.nsid);
			if (auth == null) this.authorize();
			else rc.setAuth(auth);
		}
		
        String fileName = "Photometadata-" + System.currentTimeMillis() + ".xml";
        File tempFile = new File(tempDir, fileName);
        File destFile = new File(metadataDir, fileName);
        
        Properties props = OutputPropertiesFactory.getDefaultMethodProperties(Method.XML);
        Serializer ser = SerializerFactory.getSerializer(props);
        OutputStream out = new FileOutputStream(tempFile);
        ser.setOutputStream(out );
        ContentHandler xmlSerializer = ser.asContentHandler();
        
        writeXml(xmlSerializer, out);

        if (!tempFile.renameTo(destFile)) {
            throw new IOException("Moving file failed.");
        }
        
        System.out.println("Metadata backed up successfully");
        
        downLoadPhotos();

        System.out.println("Removing temporary files");
        File[] tempFiles = tempDir.listFiles();
        for (int i = 0; i < tempFiles.length; i++) {
            File file = tempFiles[i];
            file.delete();
        }
        
        System.out.println("Backup complete");
	}
	
	private void downLoadPhotos() throws IOException {
        byte[] buffer = new byte[4096];
	    for (Map.Entry<String, String> entry : photourls.entrySet()) {
            URL url = new URL(entry.getKey());
            File file = new File(photoDir, entry.getValue());
            
            if (!file.exists()) {
                System.out.println("Downloading photo " + entry.getValue());
            
                File tempFile = new File(tempDir, entry.getValue());
                
                OutputStream out = new FileOutputStream(tempFile); 
                InputStream in = url.openStream();
                
                int count;
                while ((count = in.read(buffer)) != -1) {
                    out.write(buffer, 0, count);
                }
                out.flush();
                out.close();
                in.close();
                
                if (!tempFile.renameTo(file)) {
                    throw new IOException("Moving file failed.");
                }
            }
        }
    }

    private void writeXml(ContentHandler xmlSerializer, OutputStream out) throws SAXException, IOException {
        try {
            xmlSerializer.startDocument();
            xmlSerializer.startElement("", "rsp", "rsp", EmptyAttributes.EMPTY_ATTRIBUTES);
            
            Set<String> photoids = new TreeSet<String>();
            
            System.out.println("Retrieving photosets");
            resolver.callApi(new PhotosetListFilter(xmlSerializer, resolver, photoids), "flickr.photosets.getList", new HashMap<String, Object>());
            Map<String, Object> arguments = new HashMap<String, Object>();
            arguments.put("per_page", new Integer(500));
            System.out.println("Retrieving photos not in set");
            resolver.callApi(new NotInSetFilter(1, resolver, photoids), "flickr.photos.getNotInSet", arguments);
            
            writePhotos(photoids, xmlSerializer);
            
            xmlSerializer.endElement("", "rsp", "rsp");
        } finally {
            out.flush();
            xmlSerializer.endDocument();
        }
    }

    private void writePhotos(Set<String> photoids, ContentHandler xmlSerializer) throws SAXException {
        xmlSerializer.startElement("", "photos", "photos", EmptyAttributes.EMPTY_ATTRIBUTES);
        
        for (String id : photoids) {
            System.out.println("Retrieving photo " + id);
            Map<String, Object> arguments = new HashMap<String, Object>();
            arguments.put("photo_id", id);
            resolver.callApi(new PhotoFilter(xmlSerializer, id, resolver, photourls), "flickr.photos.getInfo", arguments);            
        }
        
        xmlSerializer.endElement("", "photos", "photos");
    }

    public static void main(String[] args) {
        try {
            Main bf = new Main(args[0], args[1], args[2], new File(args[3]));
            bf.doBackup();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
